@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 # standard libraries
 import asyncio
 import copy
@@ -9,6 +11,8 @@ import logging
 import operator
 import os
 import pathlib
+import pkgutil
+import re
 import sys
 import typing
 import uuid
@@ -40,6 +44,7 @@ from nion.swift.model import HardwareSource
 from nion.swift.model import PlugInManager
 from nion.swift.model import Profile
 from nion.ui import Application as UIApplication
+from nion.ui import CanvasItem
 from nion.ui import Declarative
 from nion.ui import Dialog
 from nion.ui import UserInterface
@@ -76,7 +81,7 @@ class Application(UIApplication.BaseApplication):
 
         self.ui.persistence_root = "3"  # sets of preferences
         self.__resources_path = resources_path
-        self.version_str = "0.15.2"
+        self.version_str = "0.15.5"
 
         self.document_model_available_event = Event.Event()
 
@@ -175,6 +180,35 @@ class Application(UIApplication.BaseApplication):
             profile, is_created = self.__establish_profile(profile_path)
         self.__profile = profile
 
+        # test code to reset script items
+        # self.__profile.script_items_updated = False
+        # while self.__profile.script_items:
+        #     self.__profile.remove_script_item(self.__profile.script_items[-1])
+
+        # migrate the interactive scripts persistent object
+        if not self.__profile.script_items_updated:
+            items_str = self.ui.get_persistent_string("interactive_scripts_1")
+            if items_str:
+                for item_dict in json.loads(items_str):
+                    item_type = item_dict.get("__type__", None)
+                    if item_type == "FolderListItem":
+                        folder_path_str = item_dict.get("full_path", None)
+                        folder_path = pathlib.Path(folder_path_str) if folder_path_str else None
+                        if folder_path:
+                            profile.append_script_item(Profile.FolderScriptItem(pathlib.Path(folder_path)))
+                    elif item_type == "ScriptListItem" and item_dict.get("indent_level", None) == 0:
+                        script_path_str = item_dict.get("full_path", None)
+                        script_path = pathlib.Path(script_path_str) if script_path_str else None
+                        if script_path:
+                            profile.append_script_item(Profile.FileScriptItem(pathlib.Path(script_path)))
+            else:
+                items_old = self.ui.get_persistent_object("interactive_scripts_0", list())
+                for script_path_str in items_old:
+                    script_path = pathlib.Path(script_path_str) if script_path_str else None
+                    if script_path:
+                        profile.append_script_item(Profile.FileScriptItem(pathlib.Path(script_path)))
+            self.__profile.script_items_updated = True
+
         # configure the document model object.
         DocumentModel.DocumentModel.computation_min_period = 0.1
         DocumentModel.DocumentModel.computation_min_factor = 1.0
@@ -239,12 +273,19 @@ class Application(UIApplication.BaseApplication):
 
         project_reference = project_reference or profile.get_project_reference(profile.last_project_reference)
 
+        update_last_project_reference = True
+
+        keyboard_modifiers = self.ui.get_keyboard_modifiers(True)
+        if keyboard_modifiers and keyboard_modifiers.shift:  # pass True since there has been no event yet
+            project_reference = None
+            update_last_project_reference = False
+
         # for backwards compatibility for beta versions. remove after limited beta sites updated.
         project_reference = project_reference or profile.get_project_reference(profile.work_project_reference_uuid)
 
         if project_reference:
             try:
-                document_controller = self.open_project_window(project_reference)
+                document_controller = self.open_project_window(project_reference, update_last_project_reference)
             except Exception:
                 self.show_ok_dialog(_("Error Opening Project"), _("Unable to open default project."), completion_fn=self.show_choose_project_dialog)
                 return True
@@ -266,7 +307,8 @@ class Application(UIApplication.BaseApplication):
             project_dialog = ProjectPanel.ProjectDialog(self.ui, self)
             project_dialog.show()
 
-    def open_project_window(self, project_reference: Profile.ProjectReference) -> DocumentController.DocumentController:
+    def open_project_window(self, project_reference: Profile.ProjectReference,
+                            update_last_project_reference: bool = True) -> DocumentController.DocumentController:
         self.__profile.read_project(project_reference)
 
         document_model = project_reference.document_model
@@ -274,9 +316,11 @@ class Application(UIApplication.BaseApplication):
         document_model.start_dispatcher()
 
         # create the document controller
-        document_controller = self.create_document_controller(document_model, "library")
+        document_controller = self.create_document_controller(document_model, "library", project_reference=project_reference)
+
+        if update_last_project_reference:
+            self.__profile.last_project_reference = project_reference.uuid
 
-        self.__profile.last_project_reference = project_reference.uuid
         project_reference.last_used = datetime.datetime.now()
 
         def window_closed():
@@ -442,9 +486,10 @@ class Application(UIApplication.BaseApplication):
         workspace_history = self.ui.get_persistent_object("workspace_history", list())
         return [pathlib.Path(file_path) for file_path in workspace_history if os.path.exists(file_path)]
 
-    def create_document_controller(self, document_model, workspace_id, display_item=None):
+    def create_document_controller(self, document_model: DocumentModel.DocumentModel, workspace_id: str, display_item=None, project_reference: typing.Optional[Profile.ProjectReference] = None) -> DocumentController.DocumentController:
         self._set_document_model(document_model)  # required to allow API to find document model
-        document_controller = DocumentController.DocumentController(self.ui, document_model, workspace_id=workspace_id, app=self)
+        document_controller = DocumentController.DocumentController(self.ui, document_model, workspace_id=workspace_id,
+                                                                    app=self, project_reference=project_reference)
         self.document_model_available_event.fire(document_model)
         # attempt to set data item / group
         if display_item:
@@ -556,6 +601,75 @@ class Application(UIApplication.BaseApplication):
         except Exception:
             self.show_ok_dialog(_("Error Opening Project"), _("Unable to open project."), completion_fn=self.show_choose_project_dialog)
 
+    @classmethod
+    def get_nion_swift_version_info(cls) -> typing.List[typing.Tuple[str, str, str]]:
+        info = list()
+        for path_str in sys.path:
+            if path_str.endswith("site-packages"):
+                site_packages_path = pathlib.Path(path_str)
+                for package in sorted(path.stem for path in site_packages_path.glob("*dist-info")):
+                    m = re.match("(.*)-(\\d+\\.\\d+.*\\d*)", package)
+                    g = m.groups() if m else list()
+                    if len(g) >= 2:
+                        package_name = g[0]
+                        package_version = g[1]
+                        if package_name.startswith("nion") or package_name in ("python", "scipy", "numpy", "h5py"):
+                            info.append((package, package_name, package_version))
+        return info
+
+    def show_about_box(self, parent_window: UIWindow.Window) -> None:
+        class AboutDialog(Dialog.OkCancelDialog):
+            def __init__(self, ui: UserInterface.UserInterface, parent_window: UIWindow.Window, version_str: str):
+                super().__init__(ui, include_cancel=False, parent_window=parent_window)
+                row = self.ui.create_row_widget()
+                logo_column = self.ui.create_column_widget()
+                logo_button = self.ui.create_push_button_widget()
+                logo_button.icon = CanvasItem.load_rgba_data_from_bytes(
+                    pkgutil.get_data(__name__, "resources/logo3.png"))
+                logo_column.add_spacing(26)
+                logo_column.add(logo_button)
+                logo_column.add_stretch()
+                column = self.ui.create_column_widget()
+
+                def make_label_row(label: str):
+                    row_one = self.ui.create_row_widget()
+                    row_one.add_spacing(13)
+                    row_one.add(self.ui.create_label_widget(label))
+                    row_one.add_spacing(13)
+                    row_one.add_stretch()
+                    return row_one
+
+                column.add_spacing(26)
+                column.add(make_label_row(f"Nion Swift {version_str}"))
+                column.add(make_label_row("Copyright 2012-2021 Nion Company. All Rights Reserved."))
+                column.add_spacing(13)
+                column.add(make_label_row(f"Python {sys.version}"))
+                if sys.base_prefix:
+                    column.add_spacing(13)
+                    column.add(make_label_row(sys.base_prefix))
+                package_name_column = self.ui.create_column_widget()
+                package_version_column = self.ui.create_column_widget()
+                for _, package_name, package_version in Application.get_nion_swift_version_info():
+                    package_name_column.add(self.ui.create_label_widget(package_name))
+                    package_version_column.add(self.ui.create_label_widget(package_version))
+                version_columns = self.ui.create_row_widget()
+                version_columns.add_spacing(13)
+                version_columns.add(package_name_column)
+                version_columns.add_spacing(8)
+                version_columns.add(package_version_column)
+                version_columns.add_spacing(13)
+                version_columns.add_stretch()
+                column.add_spacing(13)
+                column.add(version_columns)
+                column.add_spacing(13)
+                column.add_stretch()
+                row.add(logo_column)
+                row.add(column)
+                self.content.add(row)
+
+        about_dialog = AboutDialog(self.ui, parent_window, self.version_str)
+        about_dialog.show()
+
     def run_all_tests(self):
         Test.run_all_tests()
 
@@ -572,6 +686,9 @@ class NewProjectAction(UIWindow.Action):
     action_id = "project.new_project"
     action_name = _("New Project...")
 
+    def execute(self, context: UIWindow.ActionContext) -> UIWindow.ActionResult:
+        raise NotImplementedError()
+
     def invoke(self, context: UIWindow.ActionContext) -> UIWindow.ActionResult:
         context = typing.cast(DocumentController.DocumentController.ActionContext, context)
 
@@ -677,29 +794,35 @@ class NewProjectAction(UIWindow.Action):
         new_project_dialog = NewProjectDialog(application.ui, application, application.event_loop, application.profile)
         new_project_dialog.show()
 
-        return UIWindow.ActionResult.FINISHED
+        return UIWindow.ActionResult(UIWindow.ActionStatus.FINISHED)
 
 
 class OpenProjectAction(UIWindow.Action):
     action_id = "project.open_project"
     action_name = _("Open Project...")
 
+    def execute(self, context: UIWindow.ActionContext) -> UIWindow.ActionResult:
+        raise NotImplementedError()
+
     def invoke(self, context_: UIWindow.ActionContext) -> UIWindow.ActionResult:
         context = typing.cast(DocumentController.DocumentController.ActionContext, context_)
         application = typing.cast(Application, context.application)
         application.show_open_project_dialog()
-        return UIWindow.ActionResult.FINISHED
+        return UIWindow.ActionResult(UIWindow.ActionStatus.FINISHED)
 
 
 class ChooseProjectAction(UIWindow.Action):
     action_id = "project.choose_project"
     action_name = _("Choose Project...")
 
+    def execute(self, context: UIWindow.ActionContext) -> UIWindow.ActionResult:
+        raise NotImplementedError()
+
     def invoke(self, context_: UIWindow.ActionContext) -> UIWindow.ActionResult:
         context = typing.cast(DocumentController.DocumentController.ActionContext, context_)
         application = typing.cast(Application, context.application)
         application.show_choose_project_dialog()
-        return UIWindow.ActionResult.FINISHED
+        return UIWindow.ActionResult(UIWindow.ActionStatus.FINISHED)
 
 
 UIWindow.register_action(NewProjectAction())
