@@ -19,6 +19,7 @@ from nion.swift import DataItemThumbnailWidget
 from nion.swift import Inspector
 from nion.swift import MimeTypes
 from nion.swift import Undo
+from nion.swift.model import Changes
 from nion.swift.model import DataItem
 from nion.swift.model import DataStructure
 from nion.swift.model import DisplayItem
@@ -241,7 +242,7 @@ def select_computation(document_model: DocumentModel.DocumentModel, display_item
         match_items.update(display_item.graphics)
         computations = set()
         for computation in document_model.computations:
-            if computation.output_items.intersection(match_items):
+            if set(computation.output_items).intersection(match_items):
                 computations.add(computation)
         return next(iter(computations)) if len(computations) == 1 else None
     return None
@@ -368,17 +369,19 @@ class ComputationModel:
             self.__update_error_text(error_text)
         self.document_controller.queue_task(update_computation_display)
 
-    def __variable_inserted(self, index: int, variable: Symbolic.ComputationVariable) -> None:
-        self.variable_inserted_event.fire(index, variable)
-        def handle_property_changed(key: str) -> None:
-            self.__update_computation_display()
-            self.variable_property_changed_event.fire(variable)
-        self.__variable_property_changed_event_listeners[variable.uuid] = variable.property_changed_event.listen(handle_property_changed)
+    def __computation_item_inserted(self, name: str, index: int, variable: Symbolic.ComputationVariable) -> None:
+        if name == "variables":
+            self.variable_inserted_event.fire(index, variable)
+            def handle_property_changed(key: str) -> None:
+                self.__update_computation_display()
+                self.variable_property_changed_event.fire(variable)
+            self.__variable_property_changed_event_listeners[variable.uuid] = variable.property_changed_event.listen(handle_property_changed)
 
-    def __variable_removed(self, index: int, variable: Symbolic.ComputationVariable) -> None:
-        self.variable_removed_event.fire(index, variable)
-        self.__variable_property_changed_event_listeners[variable.uuid].close()
-        del self.__variable_property_changed_event_listeners[variable.uuid]
+    def __computation_item_removed(self, name: str, index: int, variable: Symbolic.ComputationVariable) -> None:
+        if name == "variables":
+            self.variable_removed_event.fire(index, variable)
+            self.__variable_property_changed_event_listeners[variable.uuid].close()
+            del self.__variable_property_changed_event_listeners[variable.uuid]
 
     # not thread safe
     def __set_display_item(self, display_item):
@@ -398,7 +401,7 @@ class ComputationModel:
             computation = self.computation
             if computation:
                 for index, variable in enumerate(computation.variables):
-                    self.__variable_removed(0, variable)
+                    self.__computation_item_removed("variables", 0, variable)
             document_model = self.document_controller.document_model
             self.__display_item = display_item
             self.__computation = select_computation(document_model, display_item)
@@ -411,13 +414,13 @@ class ComputationModel:
                     if property == "error_text":
                         self.__update_computation_display()
                 self.__computation_changed_or_mutated_event_listener = document_model.computation_updated_event.listen(computation_updated)
-                self.__computation_variable_inserted_event_listener = computation.variable_inserted_event.listen(self.__variable_inserted)
-                self.__computation_variable_removed_event_listener = computation.variable_removed_event.listen(self.__variable_removed)
+                self.__computation_variable_inserted_event_listener = computation.item_inserted_event.listen(self.__computation_item_inserted)
+                self.__computation_variable_removed_event_listener = computation.item_removed_event.listen(self.__computation_item_removed)
                 self.__computation_property_changed_event_listener = computation.property_changed_event.listen(property_changed)
             self.__update_computation_display()
             if computation:
                 for index, variable in enumerate(computation.variables):
-                    self.__variable_inserted(index, variable)
+                    self.__computation_item_inserted("variables", index, variable)
 
 
 class ChangeVariableBinding(Binding.PropertyBinding):
@@ -1021,8 +1024,8 @@ class GraphicHandler:
                 display_item = graphic.container
                 index = 1 if property == "center_x" else 0
                 graphic_name = "rectangle"
-                center_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "center", title=_("Change {} Center").format(graphic_name), command_id="change_" + graphic_name + "_center")
-                return Inspector.CalibratedValueBinding(index, display_item, ClosingTuplePropertyBinding(center_model, "value", index))
+                property_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "center", title=_("Change {} Center").format(graphic_name), command_id="change_" + graphic_name + "_center")
+                return Inspector.CalibratedValueBinding(index, display_item, ClosingTuplePropertyBinding(property_model, "value", index))
             elif property in ("width", "height"):
                 graphic = source
                 display_item = graphic.container
@@ -1036,6 +1039,41 @@ class GraphicHandler:
                 graphic_name = "rectangle"
                 rotation_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "rotation", title=_("Change {} Rotation").format(graphic_name), command_id="change_" + graphic_name + "_size")
                 return ClosingPropertyBinding(rotation_model, "value", converter=Inspector.RadianToDegreeStringConverter())
+        if isinstance(source, Graphics.LineTypeGraphic):
+            if property in ("start_x", "start_y"):
+                graphic = source
+                display_item = graphic.container
+                index = 1 if property == "start_x" else 0
+                graphic_name = "line_profile"
+                property_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "start", title=_("Change {} Start").format(graphic_name), command_id="change_" + graphic_name + "_start")
+                return Inspector.CalibratedValueBinding(index, display_item, ClosingTuplePropertyBinding(property_model, "value", index))
+            if property in ("end_x", "end_y"):
+                graphic = source
+                display_item = graphic.container
+                index = 1 if property == "end_x" else 0
+                graphic_name = "line_profile"
+                property_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "end", title=_("Change {} End").format(graphic_name), command_id="change_" + graphic_name + "_end")
+                return Inspector.CalibratedValueBinding(index, display_item, ClosingTuplePropertyBinding(property_model, "value", index))
+            if property == "length":
+                graphic = source
+                display_item = graphic.container
+                graphic_name = "line_profile"
+                property_model1 = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "start", title=_("Change {} Length").format(graphic_name), command_id="change_" + graphic_name + "_length_start")
+                property_model2 = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "end", title=_("Change {} Length").format(graphic_name), command_id="change_" + graphic_name + "_length_end")
+                return Inspector.CalibratedLengthBinding(display_item, ClosingPropertyBinding(property_model1, "value"), ClosingPropertyBinding(property_model2, "value"))
+            if property == "angle":
+                graphic = source
+                display_item = graphic.container
+                graphic_name = "line_profile"
+                property_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "angle", title=_("Change {} Angle").format(graphic_name), command_id="change_" + graphic_name + "_angle")
+                return Inspector.CalibratedBinding(display_item, ClosingPropertyBinding(property_model, "value"), Inspector.RadianToDegreeStringConverter())
+        if isinstance(source, Graphics.LineProfileGraphic):
+            if property == "width":
+                graphic = source
+                display_item = graphic.container
+                graphic_name = "line_profile"
+                property_model = Inspector.GraphicPropertyCommandModel(self.document_controller, display_item, graphic, "width", title=_("Change {} Line Width").format(graphic_name), command_id="change_" + graphic_name + "_line_width")
+                return Inspector.CalibratedWidthBinding(display_item, ClosingPropertyBinding(property_model, "value"))
         return None
 
     @classmethod
@@ -1066,7 +1104,31 @@ class GraphicHandler:
                 u.create_label(text=_("Rotation (deg)")),
                 u.create_line_edit(text="@binding(graphic.rotation_deg)", width=90),
                 u.create_stretch(), spacing=12)
-            return u.create_column( position_row, size_row, rotation_row, spacing=8)
+            return u.create_column(position_row, size_row, rotation_row, spacing=8)
+        if isinstance(graphic, Graphics.LineProfileGraphic):
+            start_row = u.create_row(
+                u.create_label(text=_("X0"), width=24),
+                u.create_line_edit(text="@binding(graphic.start_x)", width=90),
+                u.create_label(text=_("Y0"), width=24),
+                u.create_line_edit(text="@binding(graphic.start_y)", width=90),
+                u.create_stretch(), spacing=12)
+            end_row = u.create_row(
+                u.create_label(text=_("X1"), width=24),
+                u.create_line_edit(text="@binding(graphic.end_x)", width=90),
+                u.create_label(text=_("Y1"), width=24),
+                u.create_line_edit(text="@binding(graphic.end_y)", width=90),
+                u.create_stretch(), spacing=12)
+            length_row = u.create_row(
+                u.create_label(text=_("Length"), width=24),
+                u.create_line_edit(text="@binding(graphic.length)", width=90),
+                u.create_label(text=_("Angle"), width=24),
+                u.create_line_edit(text="@binding(graphic.angle)", width=90),
+                u.create_stretch(), spacing=12)
+            line_width_row = u.create_row(
+                u.create_label(text=_("Width"), width=24),
+                u.create_line_edit(text="@binding(graphic.width)", width=90),
+                u.create_stretch(), spacing=12)
+            return u.create_column(start_row, end_row, length_row, line_width_row, spacing=8)
         return u.create_label(text=_("Unsupported Graphic") + f" {graphic.type}")
 
 
@@ -1184,7 +1246,7 @@ class VariableHandler:
         document_model = self.document_controller.document_model
         computation = self.computation
         variable = self.variable
-        base_items = computation.get_input_base_items(variable.name)
+        base_items = computation.get_variable_input_items(variable.name)
         display_item = None
         for base_item in base_items:
             if isinstance(base_item, DataItem.DataItem):
@@ -1225,7 +1287,7 @@ class VariableHandler:
             graphic_content = GraphicHandler.make_component_content(graphic)
             label_row = u.create_row(
                 u.create_label(text="@binding(variable.display_label)"),
-                u.create_label(text=f"#{container.items.index(item)}"),
+                u.create_label(text=f"#{container._bound_items.index(item)}"),
                 u.create_stretch(), spacing=8)
             return u.define_component(u.create_column(label_row, graphic_content, spacing=8))
         if resource_id == "structure":
@@ -1279,8 +1341,8 @@ class VariableHandler:
             return u.create_column(label, u.create_component_instance("graphic"), spacing=8)
         elif variable.variable_type == "structure":
             return u.create_column(u.create_component_instance("structure"), spacing=8)
-        elif variable.bound_items_model:
-            return u.create_column(u.create_column(items="variable.bound_items_model.items", item_component_id="graphic_item", spacing=8), spacing=8)
+        elif variable.is_list:
+            return u.create_column(u.create_column(items="variable._bound_items", item_component_id="graphic_item", spacing=8), spacing=8)
         else:
             return u.create_column(label, u.create_label(text=_("Missing") + " " + f"[{variable.variable_type}]"), spacing=8)
 
@@ -1294,16 +1356,16 @@ class ResultHandler:
     @property
     def display_item(self) -> typing.Optional[DisplayItem.DisplayItem]:
         document_model = self.document_controller.document_model
-        output_item = list(self.result.output_items)[0]
-        if isinstance(output_item, DataItem.DataItem):
-            return document_model.get_best_display_item_for_data_item(output_item)
+        result_items = self.result.output_items
+        if len(result_items) == 1 and isinstance(result_items[0], DataItem.DataItem):
+            return document_model.get_best_display_item_for_data_item(result_items[0])
         return None
 
     @classmethod
     def make_component_content(cls, result: Symbolic.ComputationOutput) -> Declarative.UIDescription:
         u = Declarative.DeclarativeUI()
         label = u.create_label(text="@binding(result.label)")
-        result_items = list(result.output_items)
+        result_items = result.output_items
         if len(result_items) == 1 and isinstance(result_items[0], DataItem.DataItem):
             data_source_chooser = {
                 "type": "data_source_chooser",
@@ -1315,6 +1377,51 @@ class ResultHandler:
         return u.create_column(label)
 
 
+class RemoveComputationCommand(Undo.UndoableCommand):
+
+    def __init__(self, document_controller: DocumentController.DocumentController, computation: Symbolic.Computation):
+        super().__init__(_("Remove Computation"))
+        self.__document_controller = document_controller
+        self.__old_workspace_layout = self.__document_controller.workspace_controller.deconstruct()
+        self.__new_workspace_layout = None
+        self.__computation_index = document_controller.document_model.computations.index(computation)
+        self.__undelete_logs: typing.List[Changes.UndeleteLog] = list()
+        self.initialize()
+
+    def close(self):
+        self.__document_controller = None
+        self.__old_workspace_layout = None
+        self.__new_workspace_layout = None
+        self.__computation_index = None
+        for undelete_log in self.__undelete_logs:
+            undelete_log.close()
+        self.__undelete_logs = None  # type: ignore
+        super().close()
+
+    def perform(self):
+        document_model = self.__document_controller.document_model
+        computation = document_model.computations[self.__computation_index]
+        self.__undelete_logs.append(document_model.remove_computation_with_log(computation))
+
+    def _get_modified_state(self):
+        return self.__document_controller.document_model.modified_state
+
+    def _set_modified_state(self, modified_state) -> None:
+        self.__document_controller.document_model.modified_state = modified_state
+
+    def _undo(self):
+        self.__new_workspace_layout = self.__document_controller.workspace_controller.deconstruct()
+        for undelete_log in reversed(self.__undelete_logs):
+            self.__document_controller.document_model.undelete_all(undelete_log)
+            undelete_log.close()
+        self.__undelete_logs.clear()
+        self.__document_controller.workspace_controller.reconstruct(self.__old_workspace_layout)
+
+    def _redo(self):
+        self.perform()
+        self.__document_controller.workspace_controller.reconstruct(self.__new_workspace_layout)
+
+
 class ComputationHandler:
     def __init__(self, document_controller: DocumentController.DocumentController, computation: Symbolic.Computation):
         self.document_controller = document_controller
@@ -1324,6 +1431,7 @@ class ComputationHandler:
         self.computation_parameters_model = ListModel.FilteredListModel(container=computation, master_items_key="variables")
         self.computation_parameters_model.filter = ListModel.PredicateFilter(lambda v: v.variable_type not in Symbolic.Computation.data_source_types)
         self.is_custom = computation.expression is not None
+        self.delete_state_model = Model.PropertyModel(0)
 
     def close(self) -> None:
         self.computation_inputs_model.close()
@@ -1331,6 +1439,18 @@ class ComputationHandler:
         self.computation_parameters_model.close()
         self.computation_parameters_model = None
 
+    def handle_delete(self, widget: Declarative.UIWidget) -> None:
+        self.delete_state_model.value = 1
+
+    def handle_confirm_delete(self, widget: Declarative.UIWidget) -> None:
+        command = RemoveComputationCommand(self.document_controller, self.computation)
+        command.perform()
+        self.document_controller.push_undo_command(command)
+        self.delete_state_model.value = 0
+
+    def handle_cancel_delete(self, widget: Declarative.UIWidget) -> None:
+        self.delete_state_model.value = 0
+
     def create_handler(self, component_id: str, container=None, item=None, **kwargs):
         if component_id == "variable":
             return VariableHandler(self.document_controller, self.computation, item)
@@ -1390,7 +1510,11 @@ class InspectComputationDialog(Declarative.WindowHandler):
 
         self.__computation_about_to_be_removed_event_listener = computation.about_to_be_removed_event.listen(remove_computation) if computation else None
 
-        self.__run_inspector(document_controller)
+        u = Declarative.DeclarativeUI()
+        main_page = u.create_column(u.create_component_instance("@binding(stack_page_model.value)"), u.create_stretch(), min_width=320 - 24)
+        window = u.create_window(main_page, title=_("Computation"), margin=12, window_style="tool")
+        self.run(window, parent_window=document_controller, persistent_id="computation_inspector")
+        self.__document_controller.register_dialog(self.window)
 
     def close(self) -> None:
         if self.__computation_about_to_be_removed_event_listener:
@@ -1399,36 +1523,38 @@ class InspectComputationDialog(Declarative.WindowHandler):
         self.__document_controller._computation_inspector = None
         super().close()
 
-    def __run_inspector(self, parent_window: Window) -> None:
-        u = Declarative.DeclarativeUI()
-        main_page = u.create_column(u.create_component_instance("@binding(stack_page_model.value)"), min_width=320 - 24)
-        window = u.create_window(main_page, title=_("Computation"), margin=12, window_style="tool")
-        self.run(window, parent_window=parent_window, persistent_id="computation_inspector")
-        self.__document_controller.register_dialog(self.window)
-
     def get_resource(self, resource_id: str, container=None, item=None) -> typing.Optional[Declarative.UIDescription]:
         if resource_id == "empty":
             u = Declarative.DeclarativeUI()
-            content = u.create_column(u.create_label(text=_("No computation.")), u.create_stretch())
+            content = u.create_column(u.create_label(text=_("No computation.")))
             component = u.define_component(content=content)
             return component
         if resource_id == "single":
             computation = self.computation_model.value
             u = Declarative.DeclarativeUI()
             label = u.create_label(text=computation.label)
-            inputs = u.create_column(items="computation_inputs_model.items", item_component_id="variable", spacing=8)
-            results = u.create_column(items="computation.results", item_component_id="result", spacing=8)
-            row = u.create_row(
+            inputs = u.create_column(items="computation_inputs_model.items", item_component_id="variable", spacing=8, size_policy_vertical="expanding")
+            results = u.create_column(items="computation.results", item_component_id="result", spacing=8, size_policy_vertical="expanding")
+            input_output_row = u.create_row(
                 u.create_column(inputs),
                 u.create_column(results),
                 spacing=12,
+                size_policy_vertical="expanding"
             )
             parameters = u.create_column(items="computation_parameters_model.items", item_component_id="variable", spacing=8)
             if sys.platform == "darwin":
-                note = u.create_row(u.create_label(text=_("Use Command+Shift+E to edit data item script.")), u.create_stretch(), visible="@binding(is_custom)")
+                note = u.create_row(u.create_label(text=_("Use Command+Shift+E to edit data item script.")), visible="@binding(is_custom)")
             else:
-                note = u.create_row(u.create_label(text=_("Use Ctrl+Shift+E to edit data item script.")), u.create_stretch(), visible="@binding(is_custom)")
-            component = u.define_component(content=u.create_column(label, row, parameters, note, spacing=12))
+                note = u.create_row(u.create_label(text=_("Use Ctrl+Shift+E to edit data item script.")), visible="@binding(is_custom)")
+            delete_row = u.create_row(u.create_stretch(), u.create_push_button(text=_("Delete"), spacing=12, on_clicked="handle_delete"))
+            delete_confirm_row = u.create_row(u.create_stretch(),
+                                              u.create_label(text=_("Are you sure?")),
+                                              u.create_push_button(text=_("Delete"), on_clicked="handle_confirm_delete"),
+                                              u.create_push_button(text=_("Cancel"), on_clicked="handle_cancel_delete"),
+                                              spacing=12)
+            delete_control_row = u.create_row(u.create_stack(delete_row, delete_confirm_row, current_index="@binding(delete_state_model.value)"))
+            controls = u.create_row(u.create_column(note, delete_control_row, u.create_stretch(), spacing=12), u.create_stretch())
+            component = u.define_component(content=u.create_column(label, u.create_column(input_output_row, parameters, controls, spacing=12), spacing=12))
             return component
         return None
 
