@@ -282,9 +282,9 @@ class DataListController:
 
         self.__selection_changed_listener = self.__selection.changed_event.listen(selection_changed)
         self.selected_indexes = list()
-        self.on_context_menu_event : typing.Optional[typing.Callable[[typing.Optional[DisplayItem.DisplayItem], int, int, int, int], bool]] = None
+        self.on_context_menu_event : typing.Optional[typing.Callable[[typing.Optional[DisplayItem.DisplayItem], typing.List[DisplayItem.DisplayItem], int, int, int, int], bool]] = None
         self.on_focus_changed : typing.Optional[typing.Callable[[bool], None]] = None
-        self.on_drag_started : typing.Optional[typing.Callable[[UserInterface.MimeData, numpy.ndarray], None]] = None
+        self.on_drag_started : typing.Optional[typing.Callable[[UserInterface.MimeData, typing.Optional[numpy.ndarray]], None]] = None
 
         # changed display items keep track of items whose content has changed
         # the content changed messages may come from a thread so have to be
@@ -360,7 +360,12 @@ class DataListController:
         if callable(self.on_context_menu_event):
             display_item_adapter = self.__display_item_adapters[index] if index is not None else None
             display_item = display_item_adapter.display_item if display_item_adapter else None
-            return self.on_context_menu_event(display_item, x, y, gx, gy)
+            display_items = list()
+            for display_item_adapter in self.selected_display_item_adapters:
+                display_item = display_item_adapter.display_item
+                if display_item:
+                    display_items.append(display_item)
+            return self.on_context_menu_event(display_item, display_items, x, y, gx, gy)
         return False
 
     def drag_started(self, index: int, x: int, y: int, modifiers: UserInterface.KeyboardModifiers) -> None:
@@ -375,7 +380,7 @@ class DataListController:
             MimeTypes.mime_data_put_display_items(mime_data, [display_item_adapter.display_item for display_item_adapter in display_item_adapters if display_item_adapter.display_item])
             thumbnail_data = self.__display_item_adapters[anchor_index].calculate_thumbnail_data()
         if mime_data:
-            if self.on_drag_started:
+            if callable(self.on_drag_started):
                 self.on_drag_started(mime_data, thumbnail_data)
 
     def __display_item_adapter_needs_update(self) -> None:
@@ -444,9 +449,9 @@ class DataGridController:
         self.on_delete_display_item_adapters : typing.Optional[typing.Callable[[typing.List[DisplayItemAdapter]], None]] = None
         self.on_key_pressed : typing.Optional[typing.Callable[[UserInterface.Key], bool]] = None
         self.on_display_item_adapter_double_clicked : typing.Optional[typing.Callable[[DisplayItemAdapter], bool]] = None
-        self.on_context_menu_event : typing.Optional[typing.Callable[[typing.Optional[DisplayItem.DisplayItem], int, int, int, int], bool]] = None
+        self.on_context_menu_event : typing.Optional[typing.Callable[[typing.Optional[DisplayItem.DisplayItem], typing.List[DisplayItem.DisplayItem], int, int, int, int], bool]] = None
         self.on_focus_changed : typing.Optional[typing.Callable[[bool], None]] = None
-        self.on_drag_started : typing.Optional[typing.Callable[[UserInterface.MimeData, numpy.ndarray], None]] = None
+        self.on_drag_started : typing.Optional[typing.Callable[[UserInterface.MimeData, typing.Optional[numpy.ndarray]], None]] = None
 
         self.__display_item_adapters : typing.List[DisplayItemAdapter] = list()
         self.__display_item_adapter_needs_update_listeners : typing.List = list()
@@ -609,7 +614,8 @@ class DataGridController:
         if callable(self.on_context_menu_event):
             display_item_adapter = self.__display_item_adapters[index] if index is not None else None
             display_item = display_item_adapter.display_item if display_item_adapter else None
-            return self.on_context_menu_event(display_item, x, y, gx, gy)
+            display_items = [display_item_adapter.display_item for display_item_adapter in self.selected_display_item_adapters]
+            return self.on_context_menu_event(display_item, display_items, x, y, gx, gy)
         return False
 
     def drag_started(self, index: int, x: int, y: int, modifiers: UserInterface.KeyboardModifiers) -> None:
@@ -663,7 +669,7 @@ class DataListWidget(Widgets.CompositeWidgetBase):
         data_list_widget.canvas_item.add_canvas_item(data_list_controller.canvas_item)
         self.content_widget.add(data_list_widget)
 
-        def data_list_drag_started(mime_data: UserInterface.MimeData, thumbnail_data: numpy.ndarray) -> None:
+        def data_list_drag_started(mime_data: UserInterface.MimeData, thumbnail_data: typing.Optional[numpy.ndarray]) -> None:
             self.drag(mime_data, thumbnail_data)
 
         data_list_controller.on_drag_started = data_list_drag_started
@@ -683,7 +689,7 @@ class DataGridWidget(Widgets.CompositeWidgetBase):
         data_grid_widget.canvas_item.add_canvas_item(data_grid_controller.canvas_item)
         self.content_widget.add(data_grid_widget)
 
-        def data_list_drag_started(mime_data: UserInterface.MimeData, thumbnail_data: numpy.ndarray) -> None:
+        def data_list_drag_started(mime_data: UserInterface.MimeData, thumbnail_data: typing.Optional[numpy.ndarray]) -> None:
             self.drag(mime_data, thumbnail_data)
 
         data_grid_controller.on_drag_started = data_list_drag_started
@@ -701,10 +707,13 @@ class DataPanel(Panel.Panel):
 
         ui = document_controller.ui
 
-        def show_context_menu(display_item: typing.Optional[DisplayItem.DisplayItem], x: int, y: int, gx: int, gy: int) -> bool:
-            menu = self.document_controller.create_context_menu()
-            action_context = self.document_controller._get_action_context_for_display_items([display_item] if display_item else [], None)
-            self.document_controller.populate_context_menu(menu, action_context)
+        def show_context_menu(display_item: typing.Optional[DisplayItem.DisplayItem], display_items: typing.List[DisplayItem.DisplayItem], x: int, y: int, gx: int, gy: int) -> bool:
+            document_controller = self.document_controller
+            menu = document_controller.create_context_menu()
+            action_context = document_controller._get_action_context_for_display_items(display_items, None)
+            document_controller.populate_context_menu(menu, action_context)
+            menu.add_separator()
+            document_controller.add_action_to_menu(menu, "item.delete", action_context)
             menu.popup(gx, gy)
             return True
 
@@ -726,14 +735,6 @@ class DataPanel(Panel.Panel):
 
         self.__selection_changed_event_listener = self.__selection.changed_event.listen(selection_changed)
 
-        def display_item_adapter_selection_changed(display_item_adapters: typing.List[DisplayItemAdapter]) -> None:
-            indexes = set()
-            for index, display_item_adapter in enumerate(self.__filtered_display_item_adapters_model.display_item_adapters):
-                if display_item_adapter in display_item_adapters:
-                    indexes.add(index)
-            self.__selection.set_multiple(indexes)
-            self.__notify_focus_changed()
-
         def focus_changed(focused: bool) -> None:
             self.focused = focused
 
@@ -756,8 +757,8 @@ class DataPanel(Panel.Panel):
         list_icon_button = CanvasItem.BitmapButtonCanvasItem(CanvasItem.load_rgba_data_from_bytes(pkgutil.get_data(__name__, "resources/list_icon_20.png")))
         grid_icon_button = CanvasItem.BitmapButtonCanvasItem(CanvasItem.load_rgba_data_from_bytes(pkgutil.get_data(__name__, "resources/grid_icon_20.png")))
 
-        list_icon_button.sizing.set_fixed_size(Geometry.IntSize(20, 20))
-        grid_icon_button.sizing.set_fixed_size(Geometry.IntSize(20, 20))
+        list_icon_button.update_sizing(list_icon_button.sizing.with_fixed_size(Geometry.IntSize(20, 20)))
+        grid_icon_button.update_sizing(grid_icon_button.sizing.with_fixed_size(Geometry.IntSize(20, 20)))
 
         button_row = CanvasItem.CanvasItemComposition()
         button_row.layout = CanvasItem.CanvasItemRowLayout(spacing=4)
