@@ -7,7 +7,6 @@ import typing
 
 # third party libraries
 import numpy
-import scipy.ndimage
 
 # local libraries
 from nion.data import Calibration
@@ -277,7 +276,7 @@ class ImageCanvasItemDelegate:
 
     def enter_key_pressed(self) -> None: ...
 
-    def cursor_changed(self, pos): ...
+    def cursor_changed(self, pos: typing.Optional[typing.Tuple[int]]) -> None: ...
 
     def update_display_properties(self, display_properties: typing.Mapping) -> None: ...
 
@@ -373,7 +372,7 @@ def calculate_origin_and_size(canvas_size, data_shape, image_canvas_mode, image_
     return image_canvas_origin, image_canvas_size
 
 
-class ImageCanvasItem(CanvasItem.LayerCanvasItem):
+class ImageCanvasItem(CanvasItem.CanvasItemComposition):
     """A canvas item to paint an image.
 
     Callers are expected to pass in a delegate.
@@ -473,6 +472,9 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
         self.__display_latency = False
 
     def close(self):
+        if self.__undo_command:
+            self.__undo_command.close()
+            self.__undo_command = None
         with self.__closing_lock:
             with self.__update_layout_handle_lock:
                 update_layout_handle = self.__update_layout_handle
@@ -972,6 +974,7 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
                     else:
                         self.delegate.add_index_to_selection(graphic_index)
             self.delegate.end_mouse_tracking(self.__undo_command)
+            self.__undo_command = None
         self.__graphic_drag_items = []
         self.__graphic_drag_item = None
         self.__graphic_part_data = {}
@@ -1191,15 +1194,10 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
                 pos_2d = self.map_widget_to_image(self.__last_mouse)
             self.delegate.cursor_changed(pos_2d)
 
-    def _inserted(self, container):
-        # make sure we get 'prepare_render' calls
-        self.register_prepare_canvas_item(self)
-
-    def _removed(self, container):
-        # turn off 'prepare_render' calls
-        self.unregister_prepare_canvas_item(self)
-
-    def prepare_render(self):
+    def _prepare_render(self):
+        # this is called before layout and repainting. it gives this display item a chance
+        # to update anything required for layout and trigger a layout before a repaint if
+        # anything has changed.
         self.prepare_display()
 
     def _repaint(self, drawing_context):
