@@ -785,8 +785,7 @@ class LineGraphHorizontalAxisTicksCanvasItem(CanvasItem.AbstractCanvasItem):
         super().__init__()
         self.__axes = None
         self.tick_height = 4
-        self.sizing.minimum_height = self.tick_height
-        self.sizing.maximum_height = self.tick_height
+        self.update_sizing(self.sizing.with_fixed_height(self.tick_height))
 
     def set_axes(self, axes):
         if not are_axes_equal(self.__axes, axes):
@@ -822,8 +821,7 @@ class LineGraphHorizontalAxisScaleCanvasItem(CanvasItem.AbstractCanvasItem):
         super().__init__()
         self.__axes = None
         self.font_size = 12
-        self.sizing.minimum_height = self.font_size + 4
-        self.sizing.maximum_height = self.font_size + 4
+        self.update_sizing(self.sizing.with_fixed_height(self.font_size + 4))
 
     def set_axes(self, axes):
         if not are_axes_equal(self.__axes, axes):
@@ -859,19 +857,18 @@ class LineGraphHorizontalAxisLabelCanvasItem(CanvasItem.AbstractCanvasItem):
         super().__init__()
         self.__axes = None
         self.font_size = 12
-        self.sizing.minimum_height = self.font_size + 4
-        self.sizing.maximum_height = self.font_size + 4
+        self.update_sizing(self.sizing.with_fixed_height(self.font_size + 4))
 
     def size_to_content(self):
         """ Size the canvas item to the proper height. """
         new_sizing = self.copy_sizing()
-        new_sizing.minimum_height = 0
-        new_sizing.maximum_height = 0
+        new_sizing._minimum_height = 0
+        new_sizing._maximum_height = 0
         axes = self.__axes
         if axes and axes.is_valid:
             if axes.x_calibration and axes.x_calibration.units:
-                new_sizing.minimum_height = self.font_size + 4
-                new_sizing.maximum_height = self.font_size + 4
+                new_sizing._minimum_height = self.font_size + 4
+                new_sizing._maximum_height = self.font_size + 4
         self.update_sizing(new_sizing)
 
     def set_axes(self, axes):
@@ -908,8 +905,7 @@ class LineGraphVerticalAxisTicksCanvasItem(CanvasItem.AbstractCanvasItem):
         super().__init__()
         self.__axes = None
         self.tick_width = 4
-        self.sizing.minimum_width = self.tick_width
-        self.sizing.maximum_width = self.tick_width
+        self.update_sizing(self.sizing.with_fixed_width(self.tick_width))
 
     def set_axes(self, axes):
         if not are_axes_equal(self.__axes, axes):
@@ -1002,8 +998,8 @@ class LineGraphVerticalAxisScaleCanvasItem(CanvasItem.AbstractCanvasItem):
         """ Size the canvas item to the proper width, the maximum of any label. """
         new_sizing = self.copy_sizing()
 
-        new_sizing.minimum_width = 0
-        new_sizing.maximum_width = 0
+        new_sizing._minimum_width = 0
+        new_sizing._maximum_width = 0
 
         axes = self.__axes
         if axes and axes.is_valid:
@@ -1012,8 +1008,8 @@ class LineGraphVerticalAxisScaleCanvasItem(CanvasItem.AbstractCanvasItem):
             y_range = axes.calibrated_value_max - axes.calibrated_value_min
             max_width = max(max_width, calculate_scientific_notation_drawing_width(ui_settings, self.__fonts, axes.y_ticker.value_label(axes.calibrated_value_max + y_range * 5)))
             max_width = max(max_width, calculate_scientific_notation_drawing_width(ui_settings, self.__fonts, axes.y_ticker.value_label(axes.calibrated_value_min - y_range * 5)))
-            new_sizing.minimum_width = max_width
-            new_sizing.maximum_width = max_width
+            new_sizing._minimum_width = max_width
+            new_sizing._maximum_width = max_width
 
         self.update_sizing(new_sizing)
 
@@ -1072,19 +1068,18 @@ class LineGraphVerticalAxisLabelCanvasItem(CanvasItem.AbstractCanvasItem):
         super().__init__()
         self.__axes = None
         self.font_size = 12
-        self.sizing.minimum_width = self.font_size + 4
-        self.sizing.maximum_width = self.font_size + 4
+        self.update_sizing(self.sizing.with_fixed_width(self.font_size + 4))
 
     def size_to_content(self):
         """ Size the canvas item to the proper width. """
         new_sizing = self.copy_sizing()
-        new_sizing.minimum_width = 0
-        new_sizing.maximum_width = 0
+        new_sizing._minimum_width = 0
+        new_sizing._maximum_width = 0
         axes = self.__axes
         if axes and axes.is_valid:
             if axes.y_calibration and axes.y_calibration.units:
-                new_sizing.minimum_width = self.font_size + 4
-                new_sizing.maximum_width = self.font_size + 4
+                new_sizing._minimum_width = self.font_size + 4
+                new_sizing._maximum_width = self.font_size + 4
         self.update_sizing(new_sizing)
 
     def set_axes(self, axes):
@@ -1127,8 +1122,6 @@ class LineGraphLegendCanvasItemDelegate:
 
     def create_move_display_layer_command(self, display_item: DisplayItem.DisplayItem, src_index: int, target_index: int) -> Undo.UndoableCommand: ...
 
-    def create_change_display_item_property_command(self, display_item: DisplayItem.DisplayItem, property_name: str, value) -> Inspector.ChangeDisplayItemPropertyCommand: ...
-
     def push_undo_command(self, command: Undo.UndoableCommand) -> None: ...
 
     def create_mime_data(self) -> UserInterface.MimeData: ...
@@ -1193,7 +1186,8 @@ class LineGraphLegendCanvasItem(CanvasItem.AbstractCanvasItem):
         effective_entries = self.__legend_entries[:]
 
         if self.__mouse_dragging and self.__entry_to_insert is not None and self.__dragging_index is not None:
-            DisplayItem.shift_display_layers(effective_entries, self.__dragging_index, self.__entry_to_insert)
+            # move legend entry from dragging_index to entry_to_insert
+            effective_entries.insert(self.__entry_to_insert, effective_entries.pop(self.__dragging_index))
 
         self.__effective_entries = effective_entries
 
@@ -1216,9 +1210,7 @@ class LineGraphLegendCanvasItem(CanvasItem.AbstractCanvasItem):
 
         legend_width = text_width + border * 2 + line_height
 
-        new_sizing = self.copy_sizing()
-        new_sizing.set_fixed_width(legend_width)
-        new_sizing.set_fixed_height(legend_height)
+        new_sizing = self.copy_sizing().with_fixed_width(legend_width).with_fixed_height(legend_height)
         self.update_sizing(new_sizing)
 
     def wants_drag_event(self, mime_data: UserInterface.MimeData, x: int, y: int) -> bool:
@@ -1372,10 +1364,9 @@ class LineGraphLegendCanvasItem(CanvasItem.AbstractCanvasItem):
             if source_display_item == self.__delegate.get_display_item():
                 # if we're the source item, just shift the layers
                 if from_index != self.__entry_to_insert:
-                    new_display_layers = DisplayItem.shift_display_layers(self.__display_layers, from_index, self.__entry_to_insert)
-                    self.__entry_to_insert = None
-                    command = self.__delegate.create_change_display_item_property_command(source_display_item, "display_layers", new_display_layers)
+                    command = self.__delegate.create_move_display_layer_command(source_display_item, from_index, self.__entry_to_insert)
                     command.perform()
+                    self.__entry_to_insert = None
                     self.__delegate.push_undo_command(command)
             else:
                 # if we aren't the source item, move the display layer between display items
