@@ -55,7 +55,7 @@ class LinePlotCanvasItemDelegate:
 
     def enter_key_pressed(self) -> None: ...
 
-    def cursor_changed(self, pos): ...
+    def cursor_changed(self, pos: typing.Optional[typing.Tuple[int]]) -> None: ...
 
     def update_display_properties(self, display_properties: dict) -> None: ...
 
@@ -96,7 +96,7 @@ class LinePlotCanvasItemDelegate:
     def get_document_model(self) -> DocumentModel.DocumentModel: ...
 
 
-class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
+class LinePlotCanvasItem(CanvasItem.CanvasItemComposition):
     """Display a line plot.
 
     The layout is dependent on the axes due to the dependence on the width of the text labels in the vertical axis. The
@@ -192,7 +192,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
 
         # draw the background
         line_graph_background_canvas_item = CanvasItem.CanvasItemComposition()
-        #line_graph_background_canvas_item.sizing.minimum_aspect_ratio = 1.5  # note: no maximum aspect ratio; line plot looks nice wider.
+        #line_graph_background_canvas_item.update_sizing(line_graph_background_canvas_item.size.with_minimum_aspect_ratio(1.5))  # note: no maximum aspect ratio; line plot looks nice wider.
         line_graph_background_canvas_item.add_canvas_item(CanvasItem.BackgroundCanvasItem("#FFF"))
         line_graph_background_canvas_item.add_canvas_item(line_graph_group_canvas_item)
 
@@ -236,7 +236,9 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
         self.__pending_interval = None
 
     def close(self):
-        # call super
+        if self.__undo_command:
+            self.__undo_command.close()
+            self.__undo_command = None
         with self.__closing_lock:
             self.__closed = True
         super().close()
@@ -464,8 +466,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
             scalar_xdata_list = list()
             for xdata in xdata_list:
                 if xdata:
-                    scalar_data = Image.scalar_from_array(xdata.data)
-                    scalar_data = Image.convert_to_grayscale(scalar_data)
+                    scalar_data = Image.convert_to_grayscale(xdata.data)
                     scalar_intensity_calibration = calibration_style.get_intensity_calibration(xdata)
                     scalar_dimensional_calibrations = calibration_style.get_dimensional_calibrations(xdata.dimensional_shape, xdata.dimensional_calibrations)
                     if displayed_dimensional_calibration.units == scalar_dimensional_calibrations[-1].units and intensity_calibration.units == scalar_intensity_calibration.units:
@@ -505,24 +506,28 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
 
             colors = ('#1E90FF', "#F00", "#0F0", "#00F", "#FF0", "#0FF", "#F0F", "#888", "#800", "#080", "#008", "#CCC", "#880", "#088", "#808", "#964B00")
 
+            max_layer_count = len(self.__line_graph_stack.canvas_items)
+
             display_layers = list(self.__display_layers)
 
             if len(display_layers) == 0:
                 index = 0
                 for scalar_index, scalar_xdata in enumerate(scalar_xdata_list):
                     if scalar_xdata and scalar_xdata.is_data_1d:
-                        if index < len(self.__line_graph_stack.canvas_items):
+                        if index < max_layer_count:
                             display_layers.append({"fill_color": colors[index] if index == 0 else None, "stroke_color": colors[index] if index > 0 else None, "data_index": scalar_index})
                             index += 1
                     if scalar_xdata and scalar_xdata.is_data_2d:
-                        for row in range(min(scalar_xdata.data_shape[-1], len(self.__line_graph_stack.canvas_items))):
-                            if index < len(self.__line_graph_stack.canvas_items):
+                        for row in range(min(scalar_xdata.data_shape[-1], max_layer_count)):
+                            if index < max_layer_count:
                                 display_layers.append({"fill_color": colors[index] if index == 0 else None, "stroke_color": colors[index] if index > 0 else None, "data_index": scalar_index, "data_row": row})
                                 index += 1
 
             self.___has_valid_drawn_graph_data = False
 
-            for index, display_layer in enumerate(self.__display_layers[0:len(self.__line_graph_stack.canvas_items)]):
+            display_layer_count = len(self.__display_layers[0:max_layer_count])
+
+            for index, display_layer in enumerate(self.__display_layers[0:max_layer_count]):
                 fill_color = display_layer.get("fill_color")
                 stroke_color = display_layer.get("stroke_color")
                 data_index = display_layer.get("data_index", 0)
@@ -536,14 +541,15 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
                         if scalar_xdata.is_data_2d:
                             scalar_data = scalar_xdata.data[data_row:data_row + 1, :].reshape((scalar_xdata.dimensional_shape[-1],))
                             scalar_xdata = DataAndMetadata.new_data_and_metadata(scalar_data, intensity_calibration, [displayed_dimensional_calibration])
-                    line_graph_canvas_item = self.__line_graph_stack.canvas_items[index]
+                    line_graph_canvas_item = self.__line_graph_stack.canvas_items[display_layer_count - (index + 1)]
                     line_graph_canvas_item.set_fill_color(fill_color)
                     line_graph_canvas_item.set_stroke_color(stroke_color)
                     line_graph_canvas_item.set_axes(axes)
                     line_graph_canvas_item.set_uncalibrated_xdata(scalar_xdata)
                     self.___has_valid_drawn_graph_data = scalar_xdata is not None
 
-            for index in range(len(display_layers), len(self.__line_graph_stack.canvas_items)):
+            # clear the remaining layers
+            for index in range(len(display_layers), max_layer_count):
                 line_graph_canvas_item = self.__line_graph_stack.canvas_items[index]
                 line_graph_canvas_item.set_axes(None)
                 line_graph_canvas_item.set_uncalibrated_xdata(None)
@@ -551,7 +557,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
             legend_position = self.__legend_position
             LegendEntry = collections.namedtuple("LegendEntry", ["label", "fill_color", "stroke_color"])
             legend_entries = list()
-            for index, display_layer in enumerate(self.__display_layers[0:len(self.__line_graph_stack.canvas_items)]):
+            for index, display_layer in enumerate(self.__display_layers[0:max_layer_count]):
                 data_index = display_layer.get("data_index", None)
                 data_row = display_layer.get("data_row", None)
                 label = display_layer.get("label", str())
@@ -574,15 +580,10 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
             self.__line_graph_xdata_list = list()
             self.__update_canvas_items(LineGraphCanvasItem.LineGraphAxes(), None, None, None)
 
-    def _inserted(self, container):
-        # make sure we get 'prepare_render' calls
-        self.register_prepare_canvas_item(self)
-
-    def _removed(self, container):
-        # turn off 'prepare_render' calls
-        self.unregister_prepare_canvas_item(self)
-
-    def prepare_render(self):
+    def _prepare_render(self):
+        # this is called before layout and repainting. it gives this display item a chance
+        # to update anything required for layout and trigger a layout before a repaint if
+        # anything has changed.
         self.prepare_display()
 
     def _repaint(self, drawing_context):
@@ -750,6 +751,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
             return False
         self.end_tracking(modifiers)
         self.delegate.end_mouse_tracking(self.__undo_command)
+        self.__undo_command = None
         return False
 
     def _mouse_dragged(self, start: float, end: float, modifiers=None) -> None:
