@@ -224,7 +224,7 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
         self.__source_file_path = None
         self.__is_live = False
         self.__session_manager = None
-        self.__source_proxy = self.create_item_proxy()
+        self.__source_reference = self.create_item_reference()
         self.description_changed_event = Event.Event()
         self.item_changed_event = Event.Event()
         self.metadata_changed_event = Event.Event()  # see Metadata Note above
@@ -245,7 +245,6 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
         self.__content_changed = False
         self.__suspendable_storage_cache = None
         self.__display_data_channel_refs = set()  # display data channels referencing this data item
-        self.r_var = None
         if data is not None:
             data_and_metadata = DataAndMetadata.DataAndMetadata.from_data(data, timezone=self.timezone, timezone_offset=self.timezone_offset)
             self.__set_data_metadata_direct(data_and_metadata)
@@ -293,8 +292,6 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
         return data_item_copy
 
     def close(self) -> None:
-        self.__source_proxy.close()
-        self.__source_proxy = None
         self.__data_and_metadata = None
         super().close()
 
@@ -307,7 +304,7 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
 
     @property
     def item_specifier(self) -> Persistence.PersistentObjectSpecifier:
-        return Persistence.PersistentObjectSpecifier(item_uuid=self.uuid, context_uuid=self.project.uuid)
+        return Persistence.PersistentObjectSpecifier(item_uuid=self.uuid)
 
     def insert_model_item(self, container, name, before_index, item):
         """Insert a model item. Let this item's container do it if possible; otherwise do it directly.
@@ -443,15 +440,15 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
 
     @property
     def source(self):
-        return self.__source_proxy.item
+        return self.__source_reference.item
 
     @source.setter
     def source(self, source):
-        self.__source_proxy.item = source
+        self.__source_reference.item = source
         self.source_specifier = source.project.create_specifier(source).write() if source else None
 
     def __source_specifier_changed(self, name: str, d: typing.Dict) -> None:
-        self.__source_proxy.item_specifier = Persistence.PersistentObjectSpecifier.read(d)
+        self.__source_reference.item_specifier = Persistence.PersistentObjectSpecifier.read(d)
 
     def persistent_object_context_changed(self):
         # handle case where persistent object context is set on an item that is already under transaction.
@@ -624,20 +621,6 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
             self.__data_and_metadata.data_metadata.timezone_offset = self.timezone_offset
         self.notify_property_changed(name)
 
-    def set_r_value(self, r_var: str, *, notify_changed=True) -> None:
-        """Used to signal changes to the ref var, which are kept in document controller. ugh."""
-        self.r_var = r_var
-        self._description_changed()
-        if notify_changed:  # set to False to set the r-value at startup; avoid marking it as a change
-            self.__notify_description_changed()
-
-    @property
-    def displayed_title(self):
-        if self.r_var:
-            return "{0} ({1})".format(self.title, self.r_var)
-        else:
-            return self.title
-
     # call this when the listeners need to be updated (via data_item_content_changed).
     # Calling this method will send the data_item_content_changed method to each listener by using the method
     # data_item_changes.
@@ -890,7 +873,8 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
                     dimensional_calibrations.pop(-1)
             metadata = data_and_metadata.metadata if data_and_metadata else None
             timestamp = None  # always update when the data is modified
-            self.set_data_and_metadata(DataAndMetadata.DataAndMetadata.from_data(data, intensity_calibration, dimensional_calibrations, metadata, timestamp), data_modified)
+            data_descriptor = data_and_metadata.data_descriptor
+            self.set_data_and_metadata(DataAndMetadata.DataAndMetadata.from_data(data, intensity_calibration, dimensional_calibrations, metadata, timestamp, data_descriptor), data_modified)
 
     @property
     def intensity_calibration(self) -> Calibration.Calibration:
@@ -1068,6 +1052,8 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
             if self.persistent_object_context:
                 self.reserve_external_data("data", data_shape, data_dtype)
                 data = self.__load_data()
+                if data is None:
+                    data = numpy.zeros(data_shape, data_dtype)
                 data_shape_and_dtype = data_shape, data_dtype
                 timezone = Utility.get_local_timezone()
                 timezone_offset = Utility.TimezoneMinutesToStringConverter().convert(Utility.local_utcoffset_minutes())
@@ -1202,25 +1188,6 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
     def is_data_bool(self) -> bool:
         return self.__data_and_metadata is not None and self.__data_and_metadata.is_data_bool
 
-    @property
-    def display_data_shape(self) -> typing.Optional[typing.Tuple[int, ...]]:
-        dimensional_shape = self.dimensional_shape
-        next_dimension = 0
-        if self.is_sequence:
-            next_dimension += 1
-        if self.is_collection:
-            collection_dimension_count = self.collection_dimension_count
-            datum_dimension_count = self.datum_dimension_count
-            # next dimensions are treated as collection indexes.
-            if collection_dimension_count == 1 and datum_dimension_count == 1:
-                return tuple(dimensional_shape[next_dimension:next_dimension + collection_dimension_count + datum_dimension_count])
-            elif collection_dimension_count == 2 and datum_dimension_count == 1:
-                return tuple(dimensional_shape[next_dimension:next_dimension + collection_dimension_count])
-            else:  # default, "pick"
-                return tuple(dimensional_shape[next_dimension + collection_dimension_count:next_dimension + collection_dimension_count + datum_dimension_count])
-        else:
-            return tuple(dimensional_shape[next_dimension:])
-
     def get_data_value(self, pos):
         return self.__data_and_metadata.get_data_value(pos) if self.__data_and_metadata else None
 
