@@ -518,6 +518,43 @@ class CSVImportExportHandler(ImportExportHandler):
             numpy.savetxt(path_str, data, delimiter=', ')
 
 
+
+def build_table(display_item: DisplayItem.DisplayItem) -> typing.Tuple[typing.List[str], typing.List[numpy.ndarray]]:
+    data_items = display_item.data_items
+    assert all([data_item.is_data_1d for data_item in data_items])
+
+    def make_x_data(calibration: Calibration.Calibration, length: int) -> numpy.ndarray:
+        return numpy.linspace(calibration.offset, calibration.offset + (length - 1) * calibration.scale, length)
+
+    calibration0 = data_items[0].xdata.dimensional_calibrations[0]
+    if all([calibration0 == data_item.xdata.dimensional_calibrations[0] for data_item in data_items]):
+        length = max([data_item.xdata.data_shape[0] for data_item in data_items])
+        data_list = [make_x_data(calibration0, length)]
+        headers = [f"X ({calibration0.units or 'pixel'})"]
+        for index in range(len(display_item.display_layers)):
+            display_data_channel = display_item.get_display_layer_display_data_channel(index)
+            xdata = display_data_channel.get_calculated_display_values(True).display_data_and_metadata
+            data_list.append(xdata.intensity_calibration.convert_to_calibrated_value(xdata.data))
+            label = display_item.get_display_layer_property(index, "label") or f"Data {index}"
+            label = label + f" ({xdata.intensity_calibration.units or 'None'})"
+            headers.append(label)
+    else:
+        data_list = list()
+        headers = list()
+        for index in range(len(display_item.display_layers)):
+            display_data_channel = display_item.get_display_layer_display_data_channel(index)
+            xdata = display_data_channel.get_calculated_display_values(True).display_data_and_metadata
+            data_list.append(make_x_data(xdata.dimensional_calibrations[0], xdata.data_shape[0]))
+            data_list.append(xdata.intensity_calibration.convert_to_calibrated_value(xdata.data))
+            label = display_item.get_display_layer_property(index, "label") or f"Data {index}"
+            x_label = "X " + label + f" ({xdata.dimensional_calibrations[0].units or 'pixel'})"
+            y_label = "Y " + label + f" ({xdata.intensity_calibration.units or 'None'})"
+            headers.append(x_label)
+            headers.append(y_label)
+
+    return headers, data_list
+
+
 class CSV1ImportExportHandler(ImportExportHandler):
 
     def __init__(self, io_handler_id, name, extensions):
@@ -531,31 +568,7 @@ class CSV1ImportExportHandler(ImportExportHandler):
         return x_data and x_data.is_data_1d
 
     def write_display_item(self, ui, display_item: DisplayItem.DisplayItem, path_str: str, extension: str) -> None:
-        data_items = display_item.data_items
-        assert all([data_item.is_data_1d for data_item in data_items])
-
-        def calibrate(calibration, data):
-            return calibration.offset + data * calibration.scale
-
-        def make_x_data(calibration, length):
-            return numpy.linspace(calibration.offset, calibration.offset + (length - 1) * calibration.scale, length)
-
-        calibration = data_items[0].xdata.dimensional_calibrations[0]
-        if all([calibration == data_item.xdata.dimensional_calibrations[0] for data_item in data_items]):
-            length = max([data_item.xdata.data_shape[0] for data_item in data_items])
-            data_list = [make_x_data(calibration, length)]
-            headers = [f"X ({calibration.units or 'pixel'})"]
-            for data_item, display_layer in zip(data_items, display_item.display_layers):
-                data_list.append(calibrate(data_item.xdata.intensity_calibration, data_item.xdata.data))
-                headers.append(display_layer.get("label", f"Data {display_layer['data_index']}") + f" ({data_item.xdata.intensity_calibration.units or 'None'})")
-        else:
-            data_list = list()
-            headers = list()
-            for data_item, display_layer in zip(data_items, display_item.display_layers):
-                data_list.extend([make_x_data(data_item.dimensional_calibrations[0], data_item.xdata.data_shape[0]),
-                                  calibrate(data_item.intensity_calibration, data_item.xdata.data)])
-                headers.extend(["X " + display_layer.get("label", f"Data {display_layer['data_index']}") + f" ({data_item.xdata.dimensional_calibrations[0].units or 'pixel'})",
-                                "Y " + display_layer.get("label", f"Data {display_layer['data_index']}") + f" ({data_item.xdata.intensity_calibration.units or 'None'})"])
+        headers, data_list = build_table(display_item)
 
         newline = "\n"
         delimiter = ", "
