@@ -86,11 +86,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
     def item_specifier(self) -> Persistence.PersistentObjectSpecifier:
         return Persistence.PersistentObjectSpecifier(item_uuid=self.uuid)
 
-    def create_specifier(self, item: Persistence.PersistentObject, *, allow_partial: bool = True) -> Persistence.PersistentObjectSpecifier:
-        if item.project == self and allow_partial:
-            return Persistence.PersistentObjectSpecifier(item=item)
-        else:
-            return Persistence.PersistentObjectSpecifier(item=item, context=item.project)
+    def create_specifier(self, item: Persistence.PersistentObject) -> Persistence.PersistentObjectSpecifier:
+        return Persistence.PersistentObjectSpecifier(item=item)
 
     def insert_model_item(self, container, name, before_index, item) -> None:
         # special handling to pass on to the document model
@@ -102,49 +99,6 @@ class Project(Observable.Observable, Persistence.PersistentObject):
         assert callable(self.handle_remove_model_item)
         return self.handle_remove_model_item(container, name, item, safe=safe)
 
-    def _get_related_item(self, item_specifier: Persistence.PersistentObjectSpecifier) -> typing.Optional[Persistence.PersistentObject]:
-        if item_specifier.context_uuid is None or item_specifier.context_uuid == self.uuid:
-            item_uuid = item_specifier.item_uuid
-            data_item = self.get_item_by_uuid("data_items", item_uuid)
-            if data_item:
-                return data_item
-            display_item = self.get_item_by_uuid("display_items", item_uuid)
-            if display_item:
-                return display_item
-            connection = self.get_item_by_uuid("connections", item_uuid)
-            if connection:
-                return connection
-            data_structure = self.get_item_by_uuid("data_structures", item_uuid)
-            if data_structure:
-                return data_structure
-            computation = self.get_item_by_uuid("computations", item_uuid)
-            if computation:
-                return computation
-            for display_item in self.display_items:
-                display_data_channel = display_item.get_item_by_uuid("display_data_channels", item_uuid)
-                if display_data_channel:
-                    return display_data_channel
-                graphic = display_item.get_item_by_uuid("graphics", item_uuid)
-                if graphic:
-                    return graphic
-
-            def check_data_group(data_group: DataGroup.DataGroup, item_specifier: Persistence.PersistentObjectSpecifier) -> typing.Optional[DataGroup.DataGroup]:
-                for data_group in data_group.data_groups:
-                    if data_group.uuid == item_specifier.item_uuid:
-                        return data_group
-                    matching_data_group = check_data_group(data_group, item_specifier)
-                    if matching_data_group:
-                        return matching_data_group
-                return None
-
-            for data_group in self.data_groups:
-                if data_group.uuid == item_uuid:
-                    return data_group
-                matching_data_group = check_data_group(data_group, item_specifier)
-                if matching_data_group:
-                    return matching_data_group
-        return super()._get_related_item(item_specifier)
-
     @property
     def storage_system_path(self) -> pathlib.Path:
         return pathlib.Path(self.__storage_system.get_identifier())
@@ -256,6 +210,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     data_item.finish_reading()
                     if not self.get_item_by_uuid("data_items", data_item.uuid):
                         self.load_item("data_items", len(self.data_items), data_item)
+                    else:
+                        data_item.close()
                 for item_d in properties.get("display_items", list()):
                     display_item = DisplayItem.DisplayItem()
                     display_item.begin_reading()
@@ -263,6 +219,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     display_item.finish_reading()
                     if not self.get_item_by_uuid("display_items", display_item.uuid):
                         self.load_item("display_items", len(self.display_items), display_item)
+                    else:
+                        display_item.close()
                 for item_d in properties.get("data_structures", list()):
                     data_structure = DataStructure.DataStructure()
                     data_structure.begin_reading()
@@ -270,6 +228,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     data_structure.finish_reading()
                     if not self.get_item_by_uuid("data_structures", data_structure.uuid):
                         self.load_item("data_structures", len(self.data_structures), data_structure)
+                    else:
+                        data_structure.close()
                 for item_d in properties.get("computations", list()):
                     computation = Symbolic.Computation()
                     computation.begin_reading()
@@ -279,6 +239,9 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                         self.load_item("computations", len(self.computations), computation)
                         # TODO: handle update script and bind after reload in document model
                         computation.update_script(Project._processing_descriptions)
+                        computation.reset()
+                    else:
+                        computation.close()
                 for item_d in properties.get("connections", list()):
                     connection = Connection.connection_factory(item_d.get)
                     connection.begin_reading()
@@ -286,6 +249,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     connection.finish_reading()
                     if not self.get_item_by_uuid("connections", connection.uuid):
                         self.load_item("connections", len(self.connections), connection)
+                    else:
+                        connection.close()
                 for item_d in properties.get("data_groups", list()):
                     data_group = DataGroup.data_group_factory(item_d.get)
                     data_group.begin_reading()
@@ -293,6 +258,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     data_group.finish_reading()
                     if not self.get_item_by_uuid("data_groups", data_group.uuid):
                         self.load_item("data_groups", len(self.data_groups), data_group)
+                    else:
+                        data_group.close()
                 for item_d in properties.get("workspaces", list()):
                     workspace = WorkspaceLayout.factory(item_d.get)
                     workspace.begin_reading()
@@ -300,6 +267,8 @@ class Project(Observable.Observable, Persistence.PersistentObject):
                     workspace.finish_reading()
                     if not self.get_item_by_uuid("workspaces", workspace.uuid):
                         self.load_item("workspaces", len(self.workspaces), workspace)
+                    else:
+                        workspace.close()
                 workspace_uuid_str = properties.get("workspace_uuid", None)
                 if workspace_uuid_str:
                     self._set_persistent_property_value("workspace_uuid", uuid.UUID(workspace_uuid_str))
