@@ -38,7 +38,7 @@ class StorageHandler(abc.ABC):
     def read_properties(self) -> typing.Dict: ...
 
     @abc.abstractmethod
-    def read_data(self) -> numpy.ndarray: ...
+    def read_data(self) -> typing.Optional[numpy.ndarray]: ...
 
     @abc.abstractmethod
     def write_properties(self, properties, file_datetime: datetime.datetime): ...
