@@ -27,10 +27,11 @@ def create_memory_profile_context() -> TestContext.MemoryProfileContext:
 class TestComputationPanelClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
 
     def tearDown(self):
-        pass
+        TestContext.end_leaks(self)
 
     def test_expression_updates_when_node_is_changed(self):
         with TestContext.create_memory_context() as test_context:
@@ -174,9 +175,9 @@ class TestComputationPanelClass(unittest.TestCase):
             document_model.append_data_item(data_item1)
             data_item2 = DataItem.DataItem(numpy.zeros((10, 10)))
             document_model.append_data_item(data_item2)
-            data_item3 = DataItem.DataItem(numpy.zeros((10, )))
+            data_item3 = DataItem.DataItem(numpy.zeros((5, )))
             document_model.append_data_item(data_item3)
-            computation = document_model.create_computation("target.xdata = a.xdata[1] + x")
+            computation = document_model.create_computation("target.xdata = a.xdata[6] + x")
             variable = computation.create_input_item("a", Symbolic.make_item(data_item2))
             computation.create_variable("x", value_type="integral", value=5)
             document_model.set_data_item_computation(data_item1, computation)
@@ -216,9 +217,9 @@ class TestComputationPanelClass(unittest.TestCase):
             document_model.append_data_item(data_item1)
             data_item2 = DataItem.DataItem(numpy.zeros((10, 10)))
             document_model.append_data_item(data_item2)
-            data_item3 = DataItem.DataItem(numpy.zeros((10, )))
+            data_item3 = DataItem.DataItem(numpy.zeros((5, )))
             document_model.append_data_item(data_item3)
-            computation = document_model.create_computation("target.xdata = a.xdata[1] + x")
+            computation = document_model.create_computation("target.xdata = a.xdata[6] + x")
             variable = computation.create_input_item("a", Symbolic.make_item(data_item3))
             computation.create_variable("x", value_type="integral", value=5)
             document_model.set_data_item_computation(data_item1, computation)
@@ -282,7 +283,36 @@ class TestComputationPanelClass(unittest.TestCase):
             display_item.append_display_data_channel_for_data_item(data_item3)
 
             with contextlib.closing(ComputationPanel.InspectComputationDialog(document_controller, computation)):
-                display_item.remove_graphic(interval)
+                display_item.remove_graphic(interval).close()
+
+    def test_remove_computation_undo_redo_cycle(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.zeros((2, 2)))
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            fft_display_item = document_model.get_fft_new(display_item, data_item)
+            self.assertEqual(2, len(document_model.data_items))
+            self.assertEqual(2, len(document_model.display_items))
+            self.assertEqual(1, len(document_model.computations))
+            # remove computation
+            command = ComputationPanel.RemoveComputationCommand(document_controller, document_model.computations[0])
+            command.perform()
+            document_controller.push_undo_command(command)
+            self.assertEqual(1, len(document_model.data_items))
+            self.assertEqual(1, len(document_model.display_items))
+            self.assertEqual(0, len(document_model.computations))
+            # undo and check
+            document_controller.handle_undo()
+            self.assertEqual(2, len(document_model.data_items))
+            self.assertEqual(2, len(document_model.display_items))
+            self.assertEqual(1, len(document_model.computations))
+            # redo and check
+            document_controller.handle_redo()
+            self.assertEqual(1, len(document_model.data_items))
+            self.assertEqual(1, len(document_model.display_items))
+            self.assertEqual(0, len(document_model.computations))
 
     def disabled_test_expression_updates_when_variable_is_assigned(self):
         raise Exception()
