@@ -16,6 +16,7 @@ from nion.swift import Facade
 from nion.swift.model import DataGroup
 from nion.swift.model import DataItem
 from nion.swift.model import DisplayItem
+from nion.swift.model import DocumentModel
 from nion.swift.model import Graphics
 from nion.swift.model import Symbolic
 from nion.swift.test import TestContext
@@ -59,10 +60,11 @@ def construct_test_document(document_controller: DocumentController.DocumentCont
 class TestDocumentControllerClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
 
     def tearDown(self):
-        pass
+        TestContext.end_leaks(self)
 
     def test_delete_document_controller(self):
         with TestContext.create_memory_context() as test_context:
@@ -361,8 +363,9 @@ class TestDocumentControllerClass(unittest.TestCase):
             data = ((numpy.random.randn(2, 2) + 1) * 10).astype(numpy.int32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
-            data_item_r = document_model.assign_variable_to_data_item(data_item)
-            computed_data_item = document_controller.processing_computation("target.xdata = {0}.xdata * 2".format(data_item_r))
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            r_var = document_model.assign_variable_to_display_item(display_item)
+            computed_data_item = document_controller.processing_computation(f"target.xdata = {r_var}.xdata * 2")
             document_model.recompute_all()
             self.assertTrue(numpy.array_equal(computed_data_item.data, data*2))
 
@@ -377,10 +380,13 @@ class TestDocumentControllerClass(unittest.TestCase):
             document_model.append_data_item(data_item2)
             data_item3 = DataItem.DataItem(numpy.copy(data))
             document_model.append_data_item(data_item3)
-            data_item_r = document_model.assign_variable_to_data_item(data_item)
-            data_item_r2 = document_model.assign_variable_to_data_item(data_item2)
-            document_model.assign_variable_to_data_item(data_item3)
-            computed_data_item = document_controller.processing_computation("target.xdata = {0}.xdata * 2 + {1}.xdata".format(data_item_r, data_item_r2))
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item2 = document_model.get_display_item_for_data_item(data_item2)
+            display_item3 = document_model.get_display_item_for_data_item(data_item3)
+            r_var = document_model.assign_variable_to_display_item(display_item)
+            r_var2 = document_model.assign_variable_to_display_item(display_item2)
+            document_model.assign_variable_to_display_item(display_item3)
+            computed_data_item = document_controller.processing_computation(f"target.xdata = {r_var}.xdata * 2 + {r_var2}.xdata")
             self.assertEqual(len(document_model.get_data_item_computation(computed_data_item).variables), 2)
             document_model.recompute_all()
             self.assertTrue(numpy.array_equal(computed_data_item.data, data*2 + data))
@@ -411,7 +417,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             source_display_item = document_model.get_display_item_for_data_item(source_data_item)
             target_data_item = document_model.get_line_profile_new(source_display_item, source_display_item.data_item, None)
             self.assertIn(target_data_item, document_model.data_items)
-            source_display_item.remove_graphic(source_display_item.graphics[0])
+            source_display_item.remove_graphic(source_display_item.graphics[0]).close()
             self.assertNotIn(target_data_item, document_model.data_items)
 
     def test_delete_source_data_item_of_computation_deletes_target_data_item(self):
@@ -482,7 +488,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             self.assertIn(source_data_item, document_model.data_items)
             self.assertIn(intermediate_data_item, document_model.data_items)
             self.assertIn(target_data_item, document_model.data_items)
-            intermediate_display_item.remove_graphic(interval_region2)  # this will cascade delete target and then interval_region1
+            intermediate_display_item.remove_graphic(interval_region2).close()  # this will cascade delete target and then interval_region1
             self.assertIn(source_data_item, document_model.data_items)
             self.assertIn(intermediate_data_item, document_model.data_items)
             self.assertNotIn(target_data_item, document_model.data_items)
@@ -502,7 +508,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             target_data_item2 = document_model.get_invert_new(source_display_item, source_display_item.data_item, crop_region)
             self.assertIn(target_data_item1, document_model.data_items)
             self.assertIn(target_data_item2, document_model.data_items)
-            source_display_item.remove_graphic(crop_region)
+            source_display_item.remove_graphic(crop_region).close()
             self.assertNotIn(target_data_item1, document_model.data_items)
             self.assertNotIn(target_data_item2, document_model.data_items)
 
@@ -630,6 +636,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             context_menu_items = context_menu.items
             delete_item = next(x for x in context_menu_items if x.title.startswith("Delete Data Item"))
             delete_item.callback()
+            document_controller.periodic()
             self.assertEqual(len(document_model.data_items), 0)
 
     def test_delete_by_display_panel_context_menu_only_deletes_data_item_in_display_panel(self):
@@ -649,6 +656,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             context_menu_items = context_menu.items
             delete_item = next(x for x in context_menu_items if x.title.startswith("Delete Data Item"))
             delete_item.callback()
+            document_controller.periodic()
             self.assertEqual(1, len(document_model.data_items))
             self.assertEqual(1, len(document_controller.selection.indexes))
 
@@ -658,7 +666,6 @@ class TestDocumentControllerClass(unittest.TestCase):
             document_model = document_controller.document_model
             source_data_item = DataItem.DataItem(numpy.ones((8, 8), numpy.float32))
             document_model.append_data_item(source_data_item)
-            source_display_item = document_model.get_display_item_for_data_item(source_data_item)
             extra_data_item = DataItem.DataItem(numpy.ones((8, 8), numpy.float32))
             document_model.append_data_item(extra_data_item)
             document_controller.selection.set(0)
@@ -666,8 +673,9 @@ class TestDocumentControllerClass(unittest.TestCase):
             document_controller.selected_display_panel = None  # this would occur when user clicks in data panel
             context_menu = document_controller.create_context_menu_for_display(document_controller.selected_display_items)
             context_menu_items = context_menu.items
-            delete_item = next(x for x in context_menu_items if x.title == "Delete Display Items")
+            delete_item = next(x for x in context_menu_items if x.title.startswith("Delete Display Items"))
             delete_item.callback()
+            document_controller.periodic()
             self.assertEqual(len(document_model.data_items), 0)
             self.assertEqual(len(document_controller.selection.indexes), 0)
 
@@ -746,6 +754,7 @@ class TestDocumentControllerClass(unittest.TestCase):
             context_menu_items = context_menu.items
             delete_item = next(x for x in context_menu_items if x.title.startswith("Delete Data Item"))
             delete_item.callback()
+            document_controller.periodic()
             self.assertEqual(len(document_model.data_items), 0)
             self.assertEqual(len(document_controller.selection.indexes), 0)
             self.assertEqual(None, document_controller.selected_display_item)
@@ -771,8 +780,8 @@ class TestDocumentControllerClass(unittest.TestCase):
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            document_controller.prepare_data_item_script(do_log=False)
-            self.assertEqual(data_item.r_var, "r01")
+            document_model.assign_variable_to_display_item(display_item)
+            self.assertEqual(DocumentModel.MappedItemManager().get_item_r_var(display_item), "r01")
 
     def test_display_data_channel_when_it_is_immediately_filtered(self):
         with TestContext.create_memory_context() as test_context:
