@@ -32,10 +32,11 @@ Facade.initialize()
 class TestInspectorClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
 
     def tearDown(self):
-        pass
+        TestContext.end_leaks(self)
 
     def test_info_inspector_section_follows_title_change(self):
         with TestContext.create_memory_context() as test_context:
@@ -110,6 +111,7 @@ class TestInspectorClass(unittest.TestCase):
             size_width_binding = Inspector.CalibratedSizeBinding(0, display_item, Binding.TuplePropertyBinding(rect_graphic, "size", 0))
             size_width_binding.update_source("0.6")
             self.assertEqual(center, rect_graphic.center)
+            rect_graphic.close()
 
     def test_calibration_inspector_section_binds(self):
         with TestContext.create_memory_context() as test_context:
@@ -421,9 +423,9 @@ class TestInspectorClass(unittest.TestCase):
             display_item.display_type = "line_plot"
             self.assertEqual(2, len(display_item.display_layers))
             # check for unique data rows and fill colors
-            self.assertEqual(1, len({display_layer["data_index"] for display_layer in display_item.display_layers}))
-            self.assertEqual(2, len({display_layer["data_row"] for display_layer in display_item.display_layers}))
-            self.assertEqual(2, len({display_layer["fill_color"] for display_layer in display_item.display_layers}))
+            self.assertEqual(display_item.get_display_layer_display_data_channel(0), display_item.get_display_layer_display_data_channel(1))
+            self.assertNotEqual(display_item.get_display_layer_property(0, "data_row"), display_item.get_display_layer_property(1, "data_row"))
+            self.assertNotEqual(display_item.get_display_layer_property(0, "fill_color"), display_item.get_display_layer_property(1, "fill_color"))
             display_item.display_type = "image"
             self.assertEqual(1, len(display_item.display_layers))
 
@@ -438,9 +440,9 @@ class TestInspectorClass(unittest.TestCase):
             display_item.display_type = "line_plot"
             self.assertEqual(3, len(display_item.display_layers))
             # check for unique data rows and fill colors
-            self.assertEqual(1, len({display_layer["data_index"] for display_layer in display_item.display_layers}))
-            self.assertEqual(3, len({display_layer["data_row"] for display_layer in display_item.display_layers}))
-            self.assertEqual(3, len({display_layer["fill_color"] for display_layer in display_item.display_layers}))
+            self.assertEqual(1, len({display_item.get_display_layer_display_data_channel(i) for i in range(len(display_item.display_layers))}))
+            self.assertEqual(3, len({display_item.get_display_layer_property(i, "data_row") for i in range(len(display_item.display_layers))}))
+            self.assertEqual(3, len({display_item.get_display_layer_property(i, "fill_color") for i in range(len(display_item.display_layers))}))
 
     def test_line_plot_with_one_data_items_and_two_rows_and_two_layers_displays_inspector(self):
         with TestContext.create_memory_context() as test_context:
@@ -450,7 +452,7 @@ class TestInspectorClass(unittest.TestCase):
             data_item = DataItem.DataItem(numpy.zeros((2, 32)))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
-            display_item.add_display_layer(data_index=0, data_row=1)
+            display_item.add_display_layer_for_display_data_channel(display_item.display_data_channels[0], data_row=1)
             display_item.display_type = "line_plot"
             display_panel.set_display_panel_display_item(display_item)
             inspector_panel = typing.cast(Inspector.InspectorPanel, document_controller.find_dock_panel("inspector-panel"))
@@ -1192,14 +1194,14 @@ class TestInspectorClass(unittest.TestCase):
             new_data_item = document_model.get_invert_new(display_item, display_item.data_item)
             document_model.recompute_all()
             computation = document_model.get_data_item_computation(new_data_item)
-            self.assertEqual({data_item}, computation.get_input_base_items("src"))
+            self.assertEqual({data_item}, computation.get_variable_input_items("src"))
             command = Inspector.ChangeComputationVariableCommand(document_model, computation, computation._get_variable("src"), specified_object=None)
             command.perform()
             document_controller.push_undo_command(command)
             self.assertIsNone(computation.get_input("src"))
             self.assertTrue(document_controller._undo_stack.can_undo)
             document_controller.handle_undo()
-            self.assertEqual({data_item}, computation.get_input_base_items("src"))
+            self.assertEqual({data_item}, computation.get_variable_input_items("src"))
             document_controller.handle_redo()
             self.assertIsNone(computation.get_input("src"))
 
@@ -1303,6 +1305,101 @@ class TestInspectorClass(unittest.TestCase):
             document_controller.handle_redo()
             self.assertEqual("image", display_item.display_type)
 
+    def test_remove_display_layer_command(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.zeros((8, )))
+            data_item2 = DataItem.DataItem(numpy.zeros((8, )))
+            document_model.append_data_item(data_item)
+            document_model.append_data_item(data_item2)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item.append_display_data_channel_for_data_item(data_item2)
+            self.assertEqual(2, len(display_item.display_data_channels))
+            self.assertEqual(2, len(display_item.display_layers))
+            self.assertEqual(display_item.display_data_channels[0], display_item.display_layers[0].display_data_channel)
+            self.assertEqual(display_item.display_data_channels[1], display_item.display_layers[1].display_data_channel)
+            command = Inspector.RemoveDisplayDataChannelCommand(document_controller, display_item, display_item.display_data_channels[1])
+            command.perform()
+            document_controller.push_undo_command(command)
+            self.assertEqual(1, len(display_item.display_data_channels))
+            self.assertEqual(1, len(display_item.display_layers))
+            self.assertEqual(display_item.display_data_channels[0], display_item.display_layers[0].display_data_channel)
+            command.undo()
+            self.assertEqual(2, len(display_item.display_data_channels))
+            self.assertEqual(2, len(display_item.display_layers))
+            self.assertEqual(display_item.display_data_channels[0], display_item.display_layers[0].display_data_channel)
+            self.assertEqual(display_item.display_data_channels[1], display_item.display_layers[1].display_data_channel)
+            command.redo()
+            self.assertEqual(1, len(display_item.display_data_channels))
+            self.assertEqual(1, len(display_item.display_layers))
+            self.assertEqual(display_item.display_data_channels[0], display_item.display_layers[0].display_data_channel)
+
+    def test_change_display_layer_property_command(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.zeros((8, )))
+            data_item2 = DataItem.DataItem(numpy.zeros((8, )))
+            document_model.append_data_item(data_item)
+            document_model.append_data_item(data_item2)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item.append_display_data_channel_for_data_item(data_item2)
+            self.assertIsNone(display_item.get_display_layer_property(1, "label"))
+            command = Inspector.ChangeDisplayLayerPropertyCommand(document_model, display_item, 1, "label", "A")
+            command.perform()
+            document_controller.push_undo_command(command)
+            self.assertEqual("A", display_item.get_display_layer_property(1, "label"))
+            command.undo()
+            self.assertIsNone(display_item.get_display_layer_property(1, "label"))
+            command.redo()
+            self.assertEqual("A", display_item.get_display_layer_property(1, "label"))
+
+    def test_change_display_layer_data_command(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.zeros((8, )))
+            data_item2 = DataItem.DataItem(numpy.zeros((8, )))
+            data_item3 = DataItem.DataItem(numpy.zeros((8, )))
+            document_model.append_data_item(data_item)
+            document_model.append_data_item(data_item2)
+            document_model.append_data_item(data_item3)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item.append_display_data_channel_for_data_item(data_item2)
+            display_item.append_display_data_channel(DisplayItem.DisplayDataChannel(data_item3))
+            self.assertEqual(3, len(display_item.display_data_channels))
+            self.assertEqual(2, len(display_item.display_layers))
+            self.assertEqual(display_item.display_data_channels[1], display_item.get_display_layer_display_data_channel(1))
+            command = Inspector.ChangeDisplayLayerDisplayDataChannelCommand(document_model, display_item, 1, display_item.display_data_channels[2])
+            command.perform()
+            document_controller.push_undo_command(command)
+            self.assertEqual(display_item.display_data_channels[2], display_item.get_display_layer_display_data_channel(1))
+            command.undo()
+            self.assertEqual(display_item.display_data_channels[1], display_item.get_display_layer_display_data_channel(1))
+            command.redo()
+            self.assertEqual(display_item.display_data_channels[2], display_item.get_display_layer_display_data_channel(1))
+
+    def test_setting_display_layer_to_no_display_data_channel_and_back(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            display_panel = document_controller.selected_display_panel
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.zeros((8,)))
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_layer = typing.cast(DisplayItem.DisplayLayer, display_item.display_layers[0])
+            display_layer.display_data_channel = None
+            display_panel.set_display_panel_display_item(display_item)
+            document_controller.periodic()
+            inspector_panel = document_controller.find_dock_panel("inspector-panel")
+            display_data_channel_index_widget = inspector_panel.widget.find_widget_by_id("display_data_channel_index_widget")
+            display_data_channel_index_widget.on_editing_finished(None)
+            display_data_channel_index_widget.on_editing_finished(0)
+            display_data_channel_index_widget.on_editing_finished("")
+            display_data_channel_index_widget.on_editing_finished("A")
+            display_data_channel_index_widget.on_editing_finished(0)
+
     def test_graphic_property_binding_handles_removed_graphic(self):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
@@ -1336,7 +1433,7 @@ class TestInspectorClass(unittest.TestCase):
 
             binding = Inspector.CalibratedValueBinding(-1, display_item, Inspector.ChangeGraphicPropertyBinding(document_controller, display_item, interval2, "start"))
             with contextlib.closing(binding):
-                display_item.remove_graphic(interval)
+                display_item.remove_graphic(interval).close()
 
 
 if __name__ == '__main__':
