@@ -27,10 +27,11 @@ def create_memory_profile_context() -> TestContext.MemoryProfileContext:
 class TestLineGraphCanvasItem(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
 
     def tearDown(self):
-        pass
+        TestContext.end_leaks(self)
 
     def test_data_values_give_pretty_limits_when_auto(self):
 
@@ -44,7 +45,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
         for data_in, data_out in test_ranges:
             data_min, data_max = data_in
             expected_uncalibrated_data_min, expected_uncalibrated_data_max = data_out
-            data = numpy.zeros((16, 16), dtype=numpy.float64)
+            data = numpy.zeros((16, 16), dtype=float)
             irow, icol = numpy.ogrid[0:16, 0:16]
             data[:] = data_min + (data_max - data_min) * (irow / 15.0)
             # auto on min/max
@@ -112,7 +113,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((800, 1000))
+            display_panel.layout_immediate((640, 480))
             # run test
             self.assertEqual(document_controller.tool_mode, "pointer")
             for tool_mode in ["interval"]:
@@ -134,7 +135,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             # test
             document_controller.tool_mode = "pointer"
             display_panel.display_canvas_item.simulate_drag((240, 160), (240, 480))
@@ -155,7 +156,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             region.end = 0.95
             display_item.add_graphic(region)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             # test
             document_controller.tool_mode = "pointer"
             display_panel.display_canvas_item.simulate_drag((240, 160), (240, 480))
@@ -176,8 +177,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             region.end = 0.9
             display_item.add_graphic(region)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
-            display_panel.display_canvas_item.prepare_display()  # force layout
+            display_panel.layout_immediate((640, 480))
             # test
             document_controller.tool_mode = "pointer"
             display_panel.display_canvas_item.simulate_click((240, 320))
@@ -198,7 +198,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             self.assertAlmostEqual(axes.calibrated_data_min, 0.0)
             self.assertAlmostEqual(axes.calibrated_data_max, 80.0)
@@ -217,7 +217,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             self.assertAlmostEqual(axes.calibrated_data_min, 0.0)
             self.assertAlmostEqual(axes.calibrated_data_max, 40.0)
@@ -235,7 +235,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             drawing_context = DrawingContext.DrawingContext()
             calibrated_data_min = axes.calibrated_data_min
@@ -257,7 +257,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_item.set_display_property("y_style", "log")
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             self.assertEqual(axes.data_style, "log")
             drawing_context = DrawingContext.DrawingContext()
@@ -273,14 +273,14 @@ class TestLineGraphCanvasItem(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            data_item = DataItem.DataItem(numpy.ones((8,), numpy.float))
+            data_item = DataItem.DataItem(numpy.ones((8,), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
-            display_item.data_item.set_xdata(DataAndMetadata.DataAndMetadata(lambda: None, ((8, 0), numpy.float)))
+            display_item.data_item.set_xdata(DataAndMetadata.DataAndMetadata(lambda: None, ((8, 0), float)))
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
 
     def test_line_plot_calculates_calibrated_vs_uncalibrated_display_y_values(self):
         with TestContext.create_memory_context() as test_context:
@@ -292,10 +292,10 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.data, numpy.full((8, ), 10)))
             display_item.calibration_style_id = "pixels-top-left"
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             self.assertTrue(numpy.array_equal(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.data, numpy.ones((8, ))))
 
     def test_line_plot_handles_calibrated_vs_uncalibrated_display(self):
@@ -308,10 +308,10 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             self.assertEqual(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units, "nm")
             display_item.calibration_style_id = "pixels-top-left"
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             self.assertFalse(display_panel.display_canvas_item.line_graph_canvas_item.calibrated_xdata.dimensional_calibrations[-1].units)
 
     def test_line_plot_with_no_data_handles_clicks(self):
@@ -323,7 +323,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             display_panel.display_canvas_item.simulate_click((240, 16))
 
     def test_narrow_line_plot_with_nans_is_drawn_properly(self):
@@ -338,7 +338,7 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((100, 480))
+            display_panel.layout_immediate((640, 480))
             axes = display_panel.display_canvas_item._axes
             drawing_context = DrawingContext.DrawingContext()
             calibrated_data_min = axes.calibrated_data_min
@@ -353,43 +353,62 @@ class TestLineGraphCanvasItem(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            data_item = DataItem.DataItem(numpy.ones((100,100), numpy.float))
+            data_item = DataItem.DataItem(numpy.ones((100,100), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_item.display_type = "line_plot"
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
 
     def test_line_plot_with_too_few_layers_displays_gracefully(self):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            data_item = DataItem.DataItem(numpy.ones((8,100), numpy.float))
+            data_item = DataItem.DataItem(numpy.ones((8,100), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_item.display_type = "line_plot"
             self.assertEqual(8, len(display_item.display_layers))
-            display_item.display_layers = display_item.display_layers[0:4]
+            display_item.remove_display_layer(7).close()
+            display_item.remove_display_layer(6).close()
+            display_item.remove_display_layer(5).close()
+            display_item.remove_display_layer(4).close()
             self.assertEqual(4, len(display_item.display_layers))
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
 
     def test_line_plot_displays_gracefully_when_switching_layers(self):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            data_item = DataItem.DataItem(numpy.ones((8,100), numpy.float))
+            data_item = DataItem.DataItem(numpy.ones((8,100), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             display_item.display_type = "line_plot"
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
             display_item.display_type = None
-            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_panel.layout_immediate((640, 480))
+
+    def test_line_plot_handles_composite_layers_when_display_type_is_line_plot(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item1 = DataItem.DataItem(numpy.ones((100,), float))
+            data_item2 = DataItem.DataItem(numpy.ones((100,), float))
+            document_model.append_data_item(data_item1)
+            document_model.append_data_item(data_item2)
+            display_item1 = document_model.get_display_item_for_data_item(data_item1)
+            display_item1.append_display_data_channel_for_data_item(data_item2)
+            self.assertEqual(2, len(display_item1.display_layers))
+            display_item1.display_type = "line_plot"
+            self.assertEqual(2, len(display_item1.display_layers))
+            display_item1.display_type = None
+            self.assertEqual(2, len(display_item1.display_layers))
 
     def test_check_exponents(self):
         e = LineGraphCanvasItem.Exponenter()
