@@ -24,6 +24,7 @@ Facade.initialize()
 class TestProcessingClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.test_context = TestContext.create_memory_context()
         self.document_controller = self.test_context.create_document_controller_with_application()
         self.document_model = self.document_controller.document_model
@@ -35,6 +36,7 @@ class TestProcessingClass(unittest.TestCase):
 
     def tearDown(self):
         self.test_context.close()
+        TestContext.end_leaks(self)
 
     # test processing against 1d data. doesn't test for correctness of the processing.
     def test_processing_1d(self):
@@ -60,7 +62,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -100,7 +102,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -128,7 +130,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -166,7 +168,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -204,7 +206,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -229,7 +231,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -254,7 +256,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -280,7 +282,7 @@ class TestProcessingClass(unittest.TestCase):
         for source_data_item, fn, params in processing_list:
             data_item = fn(self.document_model.get_display_item_for_data_item(source_data_item), source_data_item)
             for name, value in params.items():
-                self.document_model.get_data_item_computation(data_item)._set_variable_value(name, value)
+                self.document_model.get_data_item_computation(data_item).set_input_value(name, value)
             display_item = self.document_model.get_display_item_for_data_item(data_item)
             self.document_model.recompute_all()
             with display_item.data_item.data_ref() as data_ref:
@@ -373,7 +375,7 @@ class TestProcessingClass(unittest.TestCase):
             if data_item:
                 computation = self.document_model.get_data_item_computation(data_item)
                 for name, value in params.items():
-                    computation._set_variable_value(name, value)
+                    computation.set_input_value(name, value)
                 display_item = self.document_model.get_display_item_for_data_item(data_item)
                 self.document_model.recompute_all()
                 with display_item.data_item.data_ref() as data_ref:
@@ -418,7 +420,7 @@ class TestProcessingClass(unittest.TestCase):
             if data_item:
                 computation = self.document_model.get_data_item_computation(data_item)
                 for name, value in params.items():
-                    computation._set_variable_value(name, value)
+                    computation.set_input_value(name, value)
                 display_item = self.document_model.get_display_item_for_data_item(data_item)
                 self.document_model.recompute_all()
                 with display_item.data_item.data_ref() as data_ref:
@@ -501,6 +503,7 @@ class TestProcessingClass(unittest.TestCase):
             data_item_rgba2_copy = copy.deepcopy(data_item_rgba2)
             # make sure the computation was not copied
             self.assertNotEqual(document_model.get_data_item_computation(data_item_rgba2), document_model.get_data_item_computation(data_item_rgba2_copy))
+            data_item_rgba2_copy.close()
 
     def test_snapshot_of_processing_should_copy_data(self):
         with TestContext.create_memory_context() as test_context:
@@ -713,7 +716,7 @@ class TestProcessingClass(unittest.TestCase):
                     # modify processing. make sure data and dependent data gets updated.
                     data_changed_ref[0] = False
                     display_changed_ref[0] = False
-                    document_model.get_data_item_computation(blurred_data_item)._set_variable_value("sigma", 0.1)
+                    document_model.get_data_item_computation(blurred_data_item).set_input_value("sigma", 0.1)
                     document_model.recompute_all()
                     self.assertTrue(data_changed_ref[0])
                     self.assertTrue(display_changed_ref[0])
@@ -783,7 +786,7 @@ class TestProcessingClass(unittest.TestCase):
         # the bug was that slice processing returned the wrong number of dimensions
         with TestContext.create_memory_context() as test_context:
             document_model = test_context.create_document_model()
-            data_item = DataItem.DataItem(numpy.zeros((32, 32, 16), numpy.float))
+            data_item = DataItem.DataItem(numpy.zeros((32, 32, 16), float))
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
             slice_data_item = document_model.get_slice_sum_new(display_item, display_item.data_item)
@@ -836,22 +839,22 @@ class TestProcessingClass(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8)) + 1) * 10).astype(numpy.uint32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
 
     def test_get_two_data_sources_handles_one_selected_data_item(self):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8)) + 1) * 10).astype(numpy.uint32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
             display_item = document_model.get_display_item_for_data_item(data_item)
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             display_panel = document_controller.selected_display_panel
             display_panel.set_display_panel_display_item(display_item)
             data_sources = document_controller._get_two_data_sources()
@@ -864,13 +867,13 @@ class TestProcessingClass(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8)) + 1) * 10).astype(numpy.uint32)
             data_item1 = DataItem.DataItem(data)
             document_model.append_data_item(data_item1)
             data_item2 = DataItem.DataItem(numpy.random.randn(8))
             document_model.append_data_item(data_item2)
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             document_controller.selected_display_panel = None  # use data panel selection
             document_controller.select_data_items_in_data_panel([data_item1, data_item2])
             data_sources = document_controller._get_two_data_sources()
@@ -888,13 +891,13 @@ class TestProcessingClass(unittest.TestCase):
                 data_item = DataItem.DataItem(numpy.random.randn(8))
                 document_model.append_data_item(data_item)
             document_controller.select_data_items_in_data_panel(document_model.data_items)
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
 
     def test_get_two_data_sources_handles_one_selected_data_item_and_one_crop_graphic(self):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8, 8)) + 1) * 10).astype(numpy.uint32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
@@ -916,7 +919,7 @@ class TestProcessingClass(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8, 8)) + 1) * 10).astype(numpy.uint32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
@@ -943,7 +946,7 @@ class TestProcessingClass(unittest.TestCase):
         with TestContext.create_memory_context() as test_context:
             document_controller = test_context.create_document_controller()
             document_model = document_controller.document_model
-            self.assertIsNone(document_controller._get_two_data_sources())
+            self.assertFalse(document_controller._get_two_data_sources())
             data = ((numpy.abs(numpy.random.randn(8, 8)) + 1) * 10).astype(numpy.uint32)
             data_item = DataItem.DataItem(data)
             document_model.append_data_item(data_item)
